#Header[data-v-0726bc76]{height:60px;width:100%;display:flex;align-items:center;justify-content:space-between;gap:50px;background-color:var(--greyLight);border-bottom:var(--greyDark) 2px solid}#Header #left[data-v-0726bc76]{display:flex;align-items:center;gap:20px}#Header #left #logoAndName[data-v-0726bc76]{display:flex;align-items:center;gap:10px}#Header #left #logoAndName #debiaiLogo[data-v-0726bc76]{margin:5px 10px 0 15px;cursor:pointer}#Header #left #logoAndName #projectName[data-v-0726bc76]{font-size:18px;font-weight:700}#Header #left #logoAndName #projectName a[data-v-0726bc76]{cursor:pointer;text-decoration:none}#Header #left #logoAndName #projectName a[data-v-0726bc76]:hover{text-decoration:underline}#right #items[data-v-0726bc76],#right[data-v-0726bc76]{display:flex;align-items:center;gap:20px}#right #items[data-v-0726bc76]{flex-direction:row;color:var(--fontColorLight);padding-left:30px}#right #items svg[data-v-0726bc76]{width:15px;height:15px;fill:var(--fontColorLight)}#right #buttonsTop[data-v-0726bc76]{display:flex;align-items:center;margin:5px}#right #buttonsTop #settings[data-v-0726bc76]{width:60px}#right #dates[data-v-0726bc76]{display:flex;justify-content:space-between;align-items:center;color:var(--fontColorLight);gap:20px}.computation-status[data-v-1070ff37]{width:100%}.computation-status .error[data-v-1070ff37],.computation-status .ongoing[data-v-1070ff37]{display:flex;flex-direction:column;justify-content:space-between;align-items:center;margin-top:.5rem;gap:1rem;width:100%}.computation-status .completed[data-v-1070ff37]{display:flex;align-items:center;justify-content:space-between;padding:.5rem 0;width:100%}.computation-status .completed h3[data-v-1070ff37]{color:green}.computation-status .completed div[data-v-1070ff37]{display:flex;align-items:center;gap:.2rem}.computation-status .progressBar[data-v-1070ff37]{width:100%;height:10px;background-color:#f0f0f0;border-radius:5px;overflow:hidden}.computation-status .progressBar .progress[data-v-1070ff37]{width:0;height:100%;background-color:#4caf50;transition:width .3s ease-in-out}.item[data-v-3ce0fcb0]{display:flex;flex-direction:column;align-items:flex-start;justify-content:flex-start;text-align:left;width:95%;margin:0}h2[data-v-3ce0fcb0]{margin:0}.description[data-v-3ce0fcb0]{margin:5px 0;color:var(--fontColorLight);white-space:pre-wrap}.actions[data-v-3ce0fcb0]{width:100%;margin-top:10px;display:flex;justify-content:flex-end;gap:10px}#Explorations[data-v-d893e708]{display:flex;flex-direction:column;align-items:center;width:100%;height:100%}#Explorations .explorations-header[data-v-d893e708]{display:flex;justify-content:space-between;align-items:center;margin-bottom:20px;width:100%;max-width:1000px;padding-top:30px}#Explorations .explorations-header h2[data-v-d893e708]{font-size:24px;font-weight:700}#Explorations .explorations[data-v-d893e708]{padding:20px;width:100%;max-width:1000px}#Explorations .explorations .no-explorations[data-v-d893e708]{text-align:center;font-size:18px;color:#666}.no-explorations[data-v-d893e708]{margin-top:20px}.exploration-form[data-v-d893e708]{display:flex;flex-direction:column;width:400px;gap:20px}.exploration-form .form-group[data-v-d893e708]{display:flex;flex-direction:column;align-items:flex-start;gap:10px}.exploration-form .form-group label[data-v-d893e708]{font-weight:700}.exploration-form .form-group input[data-v-d893e708],.exploration-form .form-group textarea[data-v-d893e708]{padding:10px;border-radius:5px;border:1px solid #ccc;width:378px}